@@ -1,45 +1,20 @@
 import sys
-from io import StringIO
 import argparse
 import json
-import os
-import datetime
-import traceback
 
 from simple_system_tests.ReportHtml import ReportHtml
 from simple_system_tests.CachedLogger import CachedLogger
 from simple_system_tests.TestCase import TestCase
-
-DEFAULT_OVERLINE_COLS=60
+from simple_system_tests.TestResult import TestResult
+from simple_system_tests.helper import *
 
 global __Suite
 global __env_params
 
-def get_overline():
-    cols = DEFAULT_OVERLINE_COLS
-    try:
-        cols = os.get_terminal_size().columns
-    except:
-        pass
-
-    ol = ""
-    for _ in range(cols):
-        ol = ol + "_"
-    return ol + "\n"
-
 def set_env_params(params):
     global __env_params
     __env_params = params
 
-def log_exception(ec):
-    exc_type, exc_value, exc_traceback = sys.exc_info()
-    tb_lines = traceback.format_exception(exc_type, exc_value, exc_traceback)
-    last = len(tb_lines) - 1
-    logger().error(tb_lines[last - 1] + tb_lines[last])
-
-def get_time():
-    return datetime.datetime.now().timestamp()
-
 class TestSuite:
     def __init__(self):
         self._report = ReportHtml()
@@ -86,73 +61,43 @@ class TestSuite:
     def __suite(self, no_suite_setup, desc):
         start = get_time()
         if not no_suite_setup:
+            test_result = TestResult("Suite " + desc)
             try:
                 print(get_overline())
-                print(desc + " of Suite\n")
+                print("Suite " + desc + "\n")
                 self.logger = self.__cached_logger.start_logging()
                 if desc == "Setup":
                     self.prepare_func()
                 else:
                     self.teardown_func()
+                test_result.result = True
             except Exception as ec:
-                log_exception(ec)
+                self.logger.error(get_exception(ec))
                 self.logger.error("ABORT: Suite " + desc + " failed")
-                self._report.add_result("Suite " + desc, self.__cached_logger.stop_logging(), False, get_time() - start, [0,0])
+                test_result.log = self.__cached_logger.stop_logging()
+                test_result.duration = get_time() - start
+                self._report.add_result(test_result)
                 self._report.finish_results(self.__report_file)
                 sys.exit(1)
 
-            self._report.add_result("Suite " + desc, self.__cached_logger.stop_logging(), True, get_time() - start, [0,0])
+            test_result.log = self.__cached_logger.stop_logging()
+            test_result.duration = get_time() - start
+            self._report.add_result(test_result)
 
     def __run_testcase(self, tc):
-        def __perform_task(func, desc):
-            start = get_time()
-            try:
-                func()
-                return [True, get_time() - start]
-            except Exception as ec:
-                log_exception(ec)
-                self.logger.error(desc + " failed.")
-                tc_failed = True
-                return [False, get_time() - start]
-
-        def __execute():
-            [res, duration] = __perform_task(tc.execute, "Testcase")
-            if tc.timeout > 0:
-                if tc.timeout < duration:
-                    self.logger.error("Testcase execution timeout (" + str(tc.timeout) + " s) exceeded taking " + '{:.5f}'.format(duration) + " s instead.")
-                    res = False
-
-            return [not res, duration]
-
-        tc_failed = True
         tc_desc = tc.get_description()
         print(get_overline())
         print("TEST " + tc_desc + ":\n\n")
+
         self.logger = self.__cached_logger.start_logging()
-        tc.logger = self.logger
+        test_result = tc.run_testcase(self.logger)
+        test_result.log = self.__cached_logger.stop_logging()
 
-        [res, duration] = __perform_task(tc.prepare, "Preparation of testcase")
-        if not res:
-            self._report.add_result(tc_desc, self.__cached_logger.stop_logging(), False, duration, [0,0])
-            self.__fail()
-            return
-
-        retries = -1
-        while retries < tc.retry and tc_failed:
-            retries = retries + 1
-            if retries > 0:
-                self.logger.info(str(retries) + ". Retry of testcase now.")
-            tc_failed, duration = __execute()
-
-        [teardown_res, _] = __perform_task(tc.teardown, "Testcase teardown")
-
-        log = self.__cached_logger.stop_logging()
-        if not tc_failed and teardown_res:
+        if test_result.result:
             self.__pass()
         else:
             self.__fail()
-
-        self._report.add_result(tc_desc, log, not tc_failed, duration, [retries, tc.retry])
+        self._report.add_result(test_result)
 
     def add_test_case(self, test_case, sub_params=[]):
         desc = test_case.get_description()
@@ -162,31 +107,20 @@ class TestSuite:
         self.__testcases.append(test_case)
 
     def execute_tests(self):
-        self.__cached_logger = CachedLogger()
-        args = self.__parser.parse_args()
-        no_suite_setup = vars(args)["no_suite_setup"]
-        params_env = vars(args)["json_system_params"]
-        self.__report_file = vars(args)["report_output"]
+        def specific_tests_chosen():
+            for tc in self.__testcases:
+                if tc.is_active(args):
+                    return True
+            return False
 
-        try:
-            params = json.loads(open(params_env).read())
-            set_env_params(params)
-        except Exception as ec:
-            print(str(ec) + ". So no parameters will be passed!")
-
-        all_inactive = True
-        for tc in self.__testcases:
-            if tc.is_active(args):
-                all_inactive = False
-                break
-
-        if self.prepare_func:
-            self.__suite(no_suite_setup, "Setup")
-
-        for tc in self.__testcases:
-            if not all_inactive and not tc.is_active(args):
-                continue
+        def read_json_env():
+            try:
+                params = json.loads(open(params.env).read())
+                set_env_params(params)
+            except Exception as ec:
+                print(str(ec) + ". So no parameters will be passed!")
 
+        def run_testcase_s(tc):
             sub_params = tc.get_sub_params()
             if sub_params != []:
                 for i in range(len(sub_params)):
@@ -195,6 +129,24 @@ class TestSuite:
             else:
                 self.__run_testcase(tc)
 
+        self.__cached_logger = CachedLogger()
+        args = self.__parser.parse_args()
+        no_suite_setup = vars(args)["no_suite_setup"]
+        params_env = vars(args)["json_system_params"]
+        self.__report_file = vars(args)["report_output"]
+
+        read_json_env()
+
+        if self.prepare_func:
+            self.__suite(no_suite_setup, "Setup")
+
+        specific_tests_set = specific_tests_chosen()
+        for tc in self.__testcases:
+            if specific_tests_set and not tc.is_active(args):
+                continue
+
+            run_testcase_s(tc)
+
         if self.teardown_func:
             self.__suite(no_suite_setup, "Teardown")
         self._report.finish_results(self.__report_file)
@@ -206,10 +158,22 @@ class TestSuite:
         if self.__fail_counter != 0:
             sys.exit(1)
 
+def __add_tc(func, sub_params=[], retry=0, timeout=-1, prepare_func=None, teardown_func=None):
+    global __Suite
+    desc = func.__name__
+    desc = desc[0].upper() + desc[1:]
+    T=TestCase(desc.replace("_"," "))
+    T.retry=retry
+    T.timeout=timeout
+    T.execute_func = func
+    T.prepare_func = prepare_func
+    T.teardown_func = teardown_func
+    __Suite.add_test_case(T, sub_params)
+
 set_env_params({})
 __Suite = TestSuite()
 
-# public functions except for __add_tc
+# public functions
 def get_env():
     return __env_params
 
@@ -226,18 +190,6 @@ def teardown_suite(func):
 def logger():
     return __Suite.logger
 
-def __add_tc(func, sub_params=[], retry=0, timeout=-1, prepare_func=None, teardown_func=None):
-    global __Suite
-    desc = func.__name__
-    desc = desc[0].upper() + desc[1:]
-    T=TestCase(desc.replace("_"," "))
-    T.retry=retry
-    T.timeout=timeout
-    T.execute_func = func
-    T.prepare_func = prepare_func
-    T.teardown_func = teardown_func
-    __Suite.add_test_case(T, sub_params)
-
 def testcase(retry=0, timeout=-1, prepare_func=None, teardown_func=None):
     def testcase_(func):
         __add_tc(func, [], retry, timeout, prepare_func, teardown_func)
